@@ -7,7 +7,7 @@ import sys
 import time
 from itertools import chain
 from logging import LogRecord
-from typing import Callable, Optional, Tuple
+from typing import Callable, Optional
 
 import click
 
@@ -18,10 +18,7 @@ from tqdm import tqdm
 
 from sqlfluff.cli import EXIT_ERROR, EXIT_FAIL, EXIT_SUCCESS
 from sqlfluff.cli.autocomplete import dialect_shell_complete, shell_completion_enabled
-from sqlfluff.cli.formatters import (
-    OutputStreamFormatter,
-    format_linting_result_header,
-)
+from sqlfluff.cli.formatters import OutputStreamFormatter, format_linting_result_header
 from sqlfluff.cli.helpers import LazySequence, get_package_version
 from sqlfluff.cli.outputstream import OutputStream, make_output_stream
 
@@ -36,9 +33,9 @@ from sqlfluff.core import (
     dialect_selector,
 )
 from sqlfluff.core.config import progress_bar_configuration
-from sqlfluff.core.enums import Color, FormatType
 from sqlfluff.core.linter import LintingResult
 from sqlfluff.core.plugin.host import get_plugin_manager
+from sqlfluff.core.types import Color, FormatType
 
 
 class StreamHandlerTqdm(logging.StreamHandler):
@@ -303,6 +300,11 @@ def core_options(f: Callable) -> Callable:
         default=None,
         help="Set this flag to ignore inline noqa comments.",
     )(f)
+    f = click.option(
+        "--disable-noqa-except",
+        default=None,
+        help="Ignore all but the listed rules inline noqa comments.",
+    )(f)
     f = click.option(
         "--library-path",
         default=None,
@@ -398,10 +400,6 @@ def get_config(
             )
             sys.exit(EXIT_ERROR)
 
-    from_root_kwargs = {}
-    if "require_dialect" in kwargs:
-        from_root_kwargs["require_dialect"] = kwargs.pop("require_dialect")
-
     library_path = kwargs.pop("library_path", None)
 
     if not kwargs.get("warn_unused_ignores", True):
@@ -422,7 +420,7 @@ def get_config(
             extra_config_path=extra_config_path,
             ignore_local_config=ignore_local_config,
             overrides=overrides,
-            **from_root_kwargs,
+            require_dialect=kwargs.pop("require_dialect", True),
         )
     except SQLFluffUserError as err:  # pragma: no cover
         click.echo(
@@ -436,8 +434,10 @@ def get_config(
 
 
 def get_linter_and_formatter(
-    cfg: FluffConfig, output_stream: Optional[OutputStream] = None
-) -> Tuple[Linter, OutputStreamFormatter]:
+    cfg: FluffConfig,
+    output_stream: Optional[OutputStream] = None,
+    show_lint_violations: bool = False,
+) -> tuple[Linter, OutputStreamFormatter]:
     """Get a linter object given a config."""
     try:
         # We're just making sure it exists at this stage.
@@ -453,17 +453,23 @@ def get_linter_and_formatter(
         nocolor=cfg.get("nocolor"),
         verbosity=cfg.get("verbose"),
         output_line_length=cfg.get("output_line_length"),
+        show_lint_violations=show_lint_violations,
     )
     return Linter(config=cfg, formatter=formatter), formatter
 
 
 @click.group(
     context_settings={"help_option_names": ["-h", "--help"]},
-    epilog="""\b\bExamples:\n
-  sqlfluff lint --dialect postgres .\n
-  sqlfluff lint --dialect postgres --rules ST05 .\n
-  sqlfluff fix --dialect sqlite --rules LT10,ST05 src/queries\n
-  sqlfluff parse --dialect sqlite --templater jinja src/queries/common.sql
+    # NOTE: The code-block directive here looks a little odd in the CLI
+    # but is a good balance between what appears in the CLI and what appears
+    # in the auto generated docs for the CLI by sphinx.
+    epilog="""Examples:\n
+.. code-block:: sh
+
+   sqlfluff lint --dialect postgres .\n
+   sqlfluff lint --dialect mysql --rules ST05 my_query.sql\n
+   sqlfluff fix --dialect sqlite --rules LT10,ST05 src/queries\n
+   sqlfluff parse --dialect duckdb --templater jinja path/my_query.sql\n\n
 """,
 )
 @click.version_option()
@@ -574,7 +580,7 @@ def dump_file_payload(filename: Optional[str], payload: str) -> None:
 )
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def lint(
-    paths: Tuple[str],
+    paths: tuple[str],
     format: str,
     write_output: Optional[str],
     annotation_level: str,
@@ -657,7 +663,11 @@ def lint(
     if format == FormatType.json.value:
         file_output = json.dumps(result.as_records())
     elif format == FormatType.yaml.value:
-        file_output = yaml.dump(result.as_records(), sort_keys=False)
+        file_output = yaml.dump(
+            result.as_records(),
+            sort_keys=False,
+            allow_unicode=True,
+        )
     elif format == FormatType.none.value:
         file_output = ""
     elif format == FormatType.github_annotation.value:
@@ -678,7 +688,7 @@ def lint(
                         "start_line": violation["start_line_no"],
                         "start_column": violation["start_line_pos"],
                         # NOTE: There should always be a start, there _may_ not be an
-                        # end, so in that case we default back to just re-using
+                        # end, so in that case we default back to just reusing
                         # the start.
                         "end_line": violation.get(
                             "end_line_no", violation["start_line_no"]
@@ -706,6 +716,11 @@ def lint(
         github_result_native = []
         for record in result.as_records():
             filepath = record["filepath"]
+
+            # Add a group, titled with the filename
+            if record["violations"]:
+                github_result_native.append(f"::group::{filepath}")
+
             for violation in record["violations"]:
                 # NOTE: The output format is designed for GitHub action:
                 # https://docs.github.com/en/actions/using-workflows/workflow-commands-for-github-actions#setting-a-notice-message
@@ -732,6 +747,10 @@ def lint(
 
                 github_result_native.append(line)
 
+            # Close the group
+            if record["violations"]:
+                github_result_native.append("::endgroup::")
+
         file_output = "\n".join(github_result_native)
 
     if file_output:
@@ -754,7 +773,9 @@ def lint(
     if not nofail:
         if not non_human_output:
             formatter.completion_message()
-        sys.exit(result.stats(EXIT_FAIL, EXIT_SUCCESS)["exit code"])
+        exit_code = result.stats(EXIT_FAIL, EXIT_SUCCESS)["exit code"]
+        assert isinstance(exit_code, int), "result.stats error code must be integer."
+        sys.exit(exit_code)
     else:
         sys.exit(EXIT_SUCCESS)
 
@@ -907,7 +928,7 @@ def _paths_fix(
         if check and formatter.verbosity >= 0:
             click.echo("==== fixing violations ====")
 
-        click.echo(f"{num_fixable} " "fixable linting violations found")
+        click.echo(f"{num_fixable} fixable linting violations found")
 
         if check:
             click.echo(
@@ -1034,7 +1055,7 @@ def _paths_fix(
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def fix(
     force: bool,
-    paths: Tuple[str],
+    paths: tuple[str],
     check: bool = False,
     bench: bool = False,
     quiet: bool = False,
@@ -1073,7 +1094,9 @@ def fix(
     output_stream = make_output_stream(
         config, None, os.devnull if fixing_stdin else None
     )
-    lnt, formatter = get_linter_and_formatter(config, output_stream)
+    lnt, formatter = get_linter_and_formatter(
+        config, output_stream, show_lint_violations
+    )
 
     verbose = config.get("verbose")
     progress_bar_configuration.disable_progress_bar = disable_progress_bar
@@ -1131,7 +1154,7 @@ def fix(
 )
 @click.argument("paths", nargs=-1, type=click.Path(allow_dash=True))
 def cli_format(
-    paths: Tuple[str],
+    paths: tuple[str],
     bench: bool = False,
     fixed_suffix: str = "",
     logger: Optional[logging.Logger] = None,
@@ -1387,7 +1410,11 @@ def parse(
             # For yaml dumping always dump double quoted strings if they contain
             # tabs or newlines.
             yaml.add_representer(str, quoted_presenter)
-            file_output = yaml.dump(parsed_strings_dict, sort_keys=False)
+            file_output = yaml.dump(
+                parsed_strings_dict,
+                sort_keys=False,
+                allow_unicode=True,
+            )
         elif format == FormatType.json.value:
             file_output = json.dumps(parsed_strings_dict)
         elif format == FormatType.none.value:
