@@ -2,16 +2,12 @@
 
 import logging
 from bisect import bisect_left
+from collections.abc import Iterable, Iterator
 from typing import (
     Any,
     Callable,
-    Dict,
-    Iterable,
-    Iterator,
-    List,
     NamedTuple,
     Optional,
-    Tuple,
     TypeVar,
 )
 
@@ -125,12 +121,12 @@ class RawSliceBlockInfo(NamedTuple):
 
     # Given a raw file slace, return its block ID. Useful for identifying
     # regions of a file with respect to template control structures (for, if).
-    block_ids: Dict[RawFileSlice, int]
+    block_ids: dict[RawFileSlice, int]
 
     # List of block IDs that have the following characteristics:
     # - Loop body
     # - Containing only literals (no templating)
-    literal_only_loops: List[int]
+    literal_only_loops: list[int]
 
 
 class TemplatedFile:
@@ -146,8 +142,8 @@ class TemplatedFile:
         source_str: str,
         fname: str,
         templated_str: Optional[str] = None,
-        sliced_file: Optional[List[TemplatedFileSlice]] = None,
-        raw_sliced: Optional[List[RawFileSlice]] = None,
+        sliced_file: Optional[list[TemplatedFileSlice]] = None,
+        raw_sliced: Optional[list[RawFileSlice]] = None,
     ):
         """Initialise the TemplatedFile.
 
@@ -159,9 +155,9 @@ class TemplatedFile:
             fname (str): The file name.
             templated_str (Optional[str], optional): The templated string.
                 Defaults to None.
-            sliced_file (Optional[List[TemplatedFileSlice]], optional): The sliced file.
+            sliced_file (Optional[list[TemplatedFileSlice]], optional): The sliced file.
                 Defaults to None.
-            raw_sliced (Optional[List[RawFileSlice]], optional): The raw sliced file.
+            raw_sliced (Optional[list[RawFileSlice]], optional): The raw sliced file.
                 Defaults to None.
         """
         self.source_str = source_str
@@ -170,7 +166,7 @@ class TemplatedFile:
         # If no fname, we assume this is from a string or stdin.
         self.fname = fname
         # Assume that no sliced_file, means the file is not templated
-        self.sliced_file: List[TemplatedFileSlice]
+        self.sliced_file: list[TemplatedFileSlice]
         if sliced_file is None:
             if self.templated_str != self.source_str:  # pragma: no cover
                 raise ValueError("Cannot instantiate a templated file unsliced!")
@@ -184,7 +180,7 @@ class TemplatedFile:
             assert (
                 raw_sliced is None
             ), "Templated file was not sliced, but not has raw slices."
-            self.raw_sliced: List[RawFileSlice] = [
+            self.raw_sliced: list[RawFileSlice] = [
                 RawFileSlice(source_str, "literal", 0)
             ]
         else:
@@ -211,7 +207,7 @@ class TemplatedFile:
         )
 
         # Consistency check templated string and slices.
-        previous_slice = None
+        previous_slice: Optional[TemplatedFileSlice] = None
         tfs: Optional[TemplatedFileSlice] = None
         for tfs in self.sliced_file:
             if previous_slice:
@@ -254,7 +250,7 @@ class TemplatedFile:
 
     def get_line_pos_of_char_pos(
         self, char_pos: int, source: bool = True
-    ) -> Tuple[int, int]:
+    ) -> tuple[int, int]:
         """Get the line number and position of a point in the source file.
 
         Args:
@@ -285,13 +281,13 @@ class TemplatedFile:
         templated_pos: int,
         start_idx: Optional[int] = None,
         inclusive: bool = True,
-    ) -> Tuple[int, int]:
+    ) -> tuple[int, int]:
         """Find a subset of the sliced file which touch this point.
 
         NB: the last_idx is exclusive, as the intent is to use this as a slice.
         """
         start_idx = start_idx or 0
-        first_idx = None
+        first_idx: Optional[int] = None
         last_idx = start_idx
         # Work through the sliced file, starting at the start_idx if given
         # as an optimisation hint. The sliced_file is a list of TemplatedFileSlice
@@ -315,7 +311,7 @@ class TemplatedFile:
 
     def raw_slices_spanning_source_slice(
         self, source_slice: slice
-    ) -> List[RawFileSlice]:
+    ) -> list[RawFileSlice]:
         """Return a list of the raw slices spanning a set of indices."""
         # Special case: The source_slice is at the end of the file.
         last_raw_slice = self.raw_sliced[-1]
@@ -479,7 +475,7 @@ class TemplatedFile:
                     is_literal = False
         return is_literal
 
-    def source_only_slices(self) -> List[RawFileSlice]:
+    def source_only_slices(self) -> list[RawFileSlice]:
         """Return a list a slices which reference the parts only in the source.
 
         All of these slices should be expected to have zero-length
@@ -493,7 +489,7 @@ class TemplatedFile:
                 ret_buff.append(elem)
         return ret_buff
 
-    def source_position_dict_from_slice(self, source_slice: slice) -> Dict[str, int]:
+    def source_position_dict_from_slice(self, source_slice: slice) -> dict[str, int]:
         """Create a source position dict from a slice."""
         start = self.get_line_pos_of_char_pos(source_slice.start, source=True)
         stop = self.get_line_pos_of_char_pos(source_slice.stop, source=True)
@@ -515,11 +511,11 @@ class RawTemplater:
 
     name = "raw"
     templater_selector = "templater"
-    config_subsection: Tuple[str, ...] = ()
+    config_subsection: tuple[str, ...] = ()
 
     def __init__(
         self,
-        override_context: Optional[Dict[str, Any]] = None,
+        override_context: Optional[dict[str, Any]] = None,
     ) -> None:
         """Placeholder init function.
 
@@ -532,7 +528,7 @@ class RawTemplater:
 
     def sequence_files(
         self,
-        fnames: List[str],
+        fnames: list[str],
         config: Optional[FluffConfig] = None,
         formatter: Optional[FormatterInterface] = None,
     ) -> Iterable[str]:
@@ -548,7 +544,7 @@ class RawTemplater:
         fname: str,
         config: Optional[FluffConfig] = None,
         formatter: Optional[FormatterInterface] = None,
-    ) -> Tuple[TemplatedFile, List[SQLTemplaterError]]:
+    ) -> tuple[TemplatedFile, list[SQLTemplaterError]]:
         """Process a string and return a TemplatedFile.
 
         Note that the arguments are enforced as keywords because Templaters
@@ -586,7 +582,7 @@ class RawTemplater:
         fname: str,
         config: Optional[FluffConfig] = None,
         formatter: Optional[FormatterInterface] = None,
-    ) -> Iterator[Tuple[TemplatedFile, List[SQLTemplaterError]]]:
+    ) -> Iterator[tuple[TemplatedFile, list[SQLTemplaterError]]]:
         """Extended version of `process` which returns multiple variants.
 
         Unless explicitly defined, this simply yields the result of .process().
@@ -602,11 +598,11 @@ class RawTemplater:
         """
         return isinstance(other, self.__class__)
 
-    def config_pairs(self) -> List[Tuple[str, str]]:
+    def config_pairs(self) -> list[tuple[str, str]]:
         """Returns info about the given templater for output by the cli.
 
         Returns:
-            List[Tuple[str, str]]: A list of tuples containing information
+            list[tuple[str, str]]: A list of tuples containing information
                 about the given templater. Each tuple contains two strings:
                 the string 'templater' and the name of the templater.
         """
@@ -616,7 +612,7 @@ class RawTemplater:
         self,
         fname: Optional[str],
         config: Optional[FluffConfig],
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         """Get the templating context from the config.
 
         This function retrieves the templating context from the config by
