@@ -1,10 +1,13 @@
 """Defines the placeholder template."""
 
 import logging
-from typing import Dict, Optional, Tuple
+from typing import Any, Optional
 
 import regex
 
+from sqlfluff.core.config import FluffConfig
+from sqlfluff.core.errors import SQLTemplaterError
+from sqlfluff.core.formatter import FormatterInterface
 from sqlfluff.core.helpers.slice import offset_slice
 from sqlfluff.core.templaters.base import (
     RawFileSlice,
@@ -20,6 +23,10 @@ templater_logger = logging.getLogger("sqlfluff.templater")
 KNOWN_STYLES = {
     # e.g. WHERE bla = :name
     "colon": regex.compile(r"(?<![:\w\x5c]):(?P<param_name>\w+)(?!:)", regex.UNICODE),
+    # e.g. SELECT :"column" FROM :table WHERE bla = :'name'
+    "colon_optional_quotes": regex.compile(
+        r"(?<!:):(?P<quotation>['\"]?)(?P<param_name>[\w_]+)\1", regex.UNICODE
+    ),
     # e.g. WHERE bla = table:name - use with caution as more prone to false positives
     "colon_nospaces": regex.compile(r"(?<!:):(?P<param_name>\w+)", regex.UNICODE),
     # e.g. WHERE bla = :2
@@ -66,26 +73,19 @@ class PlaceholderTemplater(RawTemplater):
 
     name = "placeholder"
 
-    def __init__(self, override_context=None, **kwargs):
+    def __init__(self, override_context: Optional[dict[str, Any]] = None):
         self.default_context = dict(test_value="__test__")
         self.override_context = override_context or {}
 
     # copy of the Python templater
-    def get_context(self, config) -> Dict:
+    def get_context(
+        self,
+        fname: Optional[str],
+        config: Optional[FluffConfig],
+    ) -> dict[str, Any]:
         """Get the templating context from the config."""
-        # TODO: The config loading should be done outside the templater code. Here
-        # is a silly place.
-        if config:
-            # This is now a nested section
-            loaded_context = (
-                config.get_section((self.templater_selector, self.name)) or {}
-            )
-        else:
-            loaded_context = {}
-        live_context = {}
-        live_context.update(self.default_context)
-        live_context.update(loaded_context)
-        live_context.update(self.override_context)
+        live_context = super().get_context(fname, config)
+
         if "param_regex" in live_context and "param_style" in live_context:
             raise ValueError(
                 "Either param_style or param_regex must be provided, not both"
@@ -113,8 +113,13 @@ class PlaceholderTemplater(RawTemplater):
 
     @large_file_check
     def process(
-        self, *, in_str: str, fname: str, config=None, formatter=None
-    ) -> Tuple[Optional[TemplatedFile], list]:
+        self,
+        *,
+        in_str: str,
+        fname: str,
+        config: Optional[FluffConfig] = None,
+        formatter: Optional[FormatterInterface] = None,
+    ) -> tuple[TemplatedFile, list[SQLTemplaterError]]:
         """Process a string and return a TemplatedFile.
 
         Note that the arguments are enforced as keywords
@@ -134,7 +139,7 @@ class PlaceholderTemplater(RawTemplater):
             formatter (:obj:`CallbackFormatter`): Optional object for output.
 
         """
-        context = self.get_context(config)
+        context = self.get_context(fname, config)
         template_slices = []
         raw_slices = []
         last_pos_raw, last_pos_templated = 0, 0
@@ -155,6 +160,9 @@ class PlaceholderTemplater(RawTemplater):
                 replacement = str(context[param_name])
             else:
                 replacement = param_name
+            if "quotation" in found_param.groupdict():
+                quotation = found_param["quotation"]
+                replacement = quotation + replacement + quotation
             # add the literal to the slices
             template_slices.append(
                 TemplatedFileSlice(
