@@ -13,8 +13,10 @@ BETWEEN
 BY
 CASE
 CAST
+CLONE
 COLLATE
 CONTAINS
+CORRESPONDING
 CREATE
 CROSS
 CUBE
@@ -71,6 +73,7 @@ OVER
 PARTITION
 PIVOT
 PRECEDING
+PRIMARY
 PROTO
 RANGE
 RECURSIVE
@@ -100,21 +103,25 @@ WITHIN"""
 
 # Note BigQuery doesn't have a list of Unreserved Keywords
 # so these are just ones we need to allow parsing to work
-bigquery_unreserved_keywords = """ACCOUNT
+bigquery_unreserved_keywords = """ACCESS
+ACCOUNT
 ADD
 ADMIN
 AFTER
 ALTER
 APPLY
+ASSIGNMENT
 ASSERT
 AUTO_INCREMENT
 BEGIN
 BERNOULLI
+BI_CAPACITY
 BINARY
 BINDING
 BREAK
 CACHE
 CALL
+CAPACITY
 CASCADE
 CHAIN
 CHARACTER
@@ -146,6 +153,7 @@ DOMAIN
 DOUBLE
 DROP
 ELSEIF
+ENFORCED
 ERROR
 EXCEPTION
 EXECUTE
@@ -155,6 +163,7 @@ EXPORT
 EXTENSION
 EXTERNAL
 FILE
+FILES
 FILTER
 FIRST
 FOREIGN
@@ -167,6 +176,7 @@ GRANTED
 GRANTS
 HOUR
 ILIKE
+IMMEDIATE
 IMPORTED
 IN
 INCREMENT
@@ -174,11 +184,13 @@ INDEX
 INOUT
 INSERT
 INTEGRATION
+KEY
 ITERATE
 LANGUAGE
 LARGE
 LAST
 LEAVE
+LOAD
 LOOP
 MANAGE
 MASKING
@@ -210,10 +222,12 @@ OPERATE
 OPTION
 OPTIONS
 ORDINAL
+ORGANIZATION
 OUT
 OVERLAPS
 OVERWRITE
 OWNERSHIP
+PARTITIONS
 PERCENT
 PIPE
 POLICY
@@ -222,6 +236,7 @@ PRIMARY
 PRIOR
 PRIVILEGES
 PROCEDURE
+PROJECT
 PUBLIC
 QUALIFY
 QUARTER
@@ -233,6 +248,8 @@ RENAME
 REPEAT
 REPEATABLE
 REPLACE
+REPLICA
+RESERVATION
 RESOURCE
 RESTRICT
 RETURN
@@ -247,16 +264,20 @@ SAFE
 SATURDAY
 SCHEMA
 SCHEMAS
+SEARCH
 SECOND
 SEPARATOR
 SERVER
 SEQUENCE
 SESSION_USER
+SETS
 SHARE
+SNAPSHOT
 SOURCE
 STAGE
 START
 STREAM
+STRICT
 SUNDAY
 SYSTEM
 SYSTEM_TIME
@@ -275,6 +296,7 @@ TRIGGER
 TRUNCATE
 TUESDAY
 TYPE
+UNDROP
 UNIQUE
 UNSIGNED
 UNTIL
@@ -286,6 +308,7 @@ USER
 VALUE
 VALUES
 VARYING
+VECTOR
 VERSION
 VIEW
 WAREHOUSE
