@@ -2,7 +2,7 @@
 
 import logging
 from dataclasses import dataclass
-from typing import List, Tuple, Type, cast
+from typing import cast
 
 from sqlfluff.core.parser import BaseSegment, RawSegment
 from sqlfluff.core.rules import LintFix, LintResult
@@ -41,7 +41,7 @@ class _RebreakIndices:
 
     @classmethod
     def from_elements(
-        cls: Type["_RebreakIndices"],
+        cls: type["_RebreakIndices"],
         elements: ReflowSequenceType,
         start_idx: int,
         dir: int,
@@ -78,7 +78,7 @@ class _RebreakLocation:
 
     @classmethod
     def from_span(
-        cls: Type["_RebreakLocation"], span: _RebreakSpan, elements: ReflowSequenceType
+        cls: type["_RebreakLocation"], span: _RebreakSpan, elements: ReflowSequenceType
     ) -> "_RebreakLocation":
         """Expand a span to a location."""
         return cls(
@@ -144,7 +144,7 @@ class _RebreakLocation:
 
 def first_create_anchor(
     elem_buff: ReflowSequenceType, loc_range: range
-) -> Tuple[RawSegment, ...]:
+) -> tuple[RawSegment, ...]:
     """Handle the potential case of an empty point with the next point with segments.
 
     While a reflow element's segments are empty, search for the next
@@ -166,7 +166,7 @@ def first_create_anchor(
 
 def identify_rebreak_spans(
     element_buffer: ReflowSequenceType, root_segment: BaseSegment
-) -> List[_RebreakSpan]:
+) -> list[_RebreakSpan]:
     """Identify areas in file to rebreak.
 
     A span here is a block, or group of blocks which have
@@ -174,7 +174,7 @@ def identify_rebreak_spans(
     as raw segments themselves or by virtue of one of their
     parent segments.
     """
-    spans: List[_RebreakSpan] = []
+    spans: list[_RebreakSpan] = []
     # We'll need at least two elements each side, so constrain
     # our range accordingly.
     for idx in range(2, len(element_buffer) - 2):
@@ -263,9 +263,93 @@ def identify_rebreak_spans(
     return spans
 
 
+def identify_keyword_rebreak_spans(
+    element_buffer: ReflowSequenceType,
+) -> list[_RebreakSpan]:
+    """Identify keyword areas in file to rebreak.
+
+    A span here is a block, or group of blocks which have explicit
+    configs for their keyword's line position.
+    """
+    spans: list[_RebreakSpan] = []
+    # We'll need at least two elements each side, so constrain
+    # our range accordingly.
+    for idx in range(2, len(element_buffer) - 2):
+        elem = element_buffer[idx]
+        # Only evaluate blocks
+        if not isinstance(elem, ReflowBlock):
+            continue
+        # Do any of its parents have config, and are we at the start
+        # of them?
+        for key in elem.keyword_line_position_configs.keys():
+            # If the element has been unset by using "None" then we want to skip adding
+            # it here.
+            if elem.keyword_line_position_configs[key].lower() == "none":
+                continue
+
+            # If we're not at the start of the segment, then pass. Some keywords might
+            # be at an index of 1 due to a leading indent so check for both 0 and 1.
+            if elem.depth_info.stack_positions[key].idx > 1:
+                continue
+
+            # Next check how deep the current element is with respect to the
+            # element which is configured. If we're operating at a deeper depth than
+            # the configuration is applied to, then this keyword cannot be the leading
+            # keyword for that segment. In that case continue, because we're not
+            # looking at the trigger keyword.
+            configured_depth = elem.depth_info.stack_hashes.index(key)
+            if elem.depth_info.stack_depth > configured_depth + 1:
+                continue
+
+            # Then make sure it's actually a keyword.
+            if not element_buffer[idx].segments or not element_buffer[idx].segments[
+                0
+            ].is_type("keyword"):
+                continue
+
+            # Can we find the end?
+            # NOTE: It's safe to look right to the end here rather than up to
+            # -2 because we're going to end up stepping back by two in the
+            # complicated cases.
+            for end_idx in range(idx, len(element_buffer)):
+                end_elem = element_buffer[end_idx]
+                final_idx = None
+
+                if not isinstance(end_elem, ReflowBlock):
+                    if any(seg.is_type("indent") for seg in end_elem.segments):
+                        final_idx = end_idx - 1
+                    else:
+                        continue
+                elif end_elem.depth_info.stack_positions[key].type in ("end", "solo"):
+                    final_idx = end_idx
+
+                if final_idx is not None:
+                    # Found the end. Add it to the stack.
+                    # We reference the appropriate element from the parent stack.
+                    target = element_buffer[idx].segments[0]
+                    spans.append(
+                        _RebreakSpan(
+                            target,
+                            idx,
+                            final_idx,
+                            # NOTE: this isn't pretty but until it needs to be more
+                            # complex, this works.
+                            elem.keyword_line_position_configs[key].split(":")[0],
+                            elem.keyword_line_position_configs[key].endswith("strict"),
+                        )
+                    )
+                    break
+            # If we find the start, but not the end, it's not a problem, but
+            # we won't be rebreaking this span. This is important so that we
+            # don't rebreak part of something without the context of what's
+            # in the rest of it. We continue without adding it to the buffer.
+    return spans
+
+
 def rebreak_sequence(
-    elements: ReflowSequenceType, root_segment: BaseSegment
-) -> Tuple[ReflowSequenceType, List[LintResult]]:
+    elements: ReflowSequenceType,
+    root_segment: BaseSegment,
+) -> tuple[ReflowSequenceType, list[LintResult]]:
     """Reflow line breaks within a sequence.
 
     Initially this only _moves_ existing segments
@@ -276,8 +360,8 @@ def rebreak_sequence(
     This intentionally does *not* handle indentation,
     as the existing indents are assumed to be correct.
     """
-    lint_results: List[LintResult] = []
-    fixes: List[LintFix] = []
+    lint_results: list[LintResult] = []
+    fixes: list[LintFix] = []
     elem_buff: ReflowSequenceType = elements.copy()
 
     # Given a sequence we should identify the objects which
@@ -294,7 +378,7 @@ def rebreak_sequence(
     # to handle comments differently. There are two other important points:
     # 1. The next newline outward before code (but passing over comments).
     # 2. The point before the next _code_ segment (ditto comments).
-    locations: List[_RebreakLocation] = []
+    locations: list[_RebreakLocation] = []
     for span in spans:
         try:
             locations.append(_RebreakLocation.from_span(span, elem_buff))
@@ -577,3 +661,190 @@ def rebreak_sequence(
         fixes = []
 
     return elem_buff, lint_results
+
+
+def rebreak_keywords_sequence(
+    elements: ReflowSequenceType,
+    root_segment: BaseSegment,
+) -> tuple[ReflowSequenceType, list[LintResult]]:
+    """Reflow line breaks within a sequence.
+
+    Initially this only _moves_ existing segments
+    around line breaks (e.g. for operators and commas),
+    but eventually this method should also handle line
+    length considerations too.
+
+    This intentionally does *not* handle indentation,
+    as the existing indents are assumed to be correct.
+    """
+    lint_results: list[LintResult] = []
+    fixes: list[LintFix] = []
+    elem_buff: ReflowSequenceType = elements.copy()
+
+    # Given a sequence we should identify the objects which
+    # make sense to rebreak. That includes any raws with config,
+    # but also and parent segments which have config and we can
+    # find both ends for. Given those spans, we then need to find
+    # the points either side of them and then the blocks either
+    # side to respace them at the same time.
+
+    # 1. First find appropriate spans.
+    spans = identify_keyword_rebreak_spans(elem_buff)
+
+    # The spans give us the edges of operators, but for line positioning we need
+    # to handle comments differently. There are two other important points:
+    # 1. The next newline outward before code (but passing over comments).
+    # 2. The point before the next _code_ segment (ditto comments).
+    locations: list[_RebreakLocation] = []
+    for span in spans:
+        try:
+            locations.append(_RebreakLocation.from_span(span, elem_buff))
+        # If we try and create a location from an incomplete span (i.e. one
+        # where we're unable to find the next newline effectively), then
+        # we'll get an exception. If we do - skip that one - we won't be
+        # able to effectively work with it even if we could construct it.
+        # This would be unlikely to happen when breaking on only keywords,
+        # but was left in that unlikely event.
+        except UnboundLocalError:  # pragma: no cover
+            pass
+
+    # Handle each span:
+    for loc in locations:
+        reflow_logger.debug(
+            "Handing Rebreak Span (%r: %s): %r",
+            loc.line_position,
+            loc.target,
+            "".join(
+                elem.raw
+                for elem in elem_buff[
+                    loc.prev.pre_code_pt_idx - 1 : loc.next.pre_code_pt_idx + 2
+                ]
+            ),
+        )
+
+        if loc.has_inappropriate_newlines(elem_buff, True):
+            continue
+
+        if loc.has_templated_newline(elem_buff):
+            continue
+
+        # Points and blocks either side are just offsets from the indices.
+        prev_point = cast(ReflowPoint, elem_buff[loc.prev.adj_pt_idx])
+        next_point = cast(ReflowPoint, elem_buff[loc.next.adj_pt_idx])
+
+        # So we know we have a preference, is it ok?
+        if loc.line_position == "leading":
+            if elem_buff[loc.prev.newline_pt_idx].num_newlines():
+                # We're good. It's already leading.
+                continue
+
+            # Generate the text for any issues.
+            pretty_name = loc.pretty_target_name()
+            desc = f"The {pretty_name} should always start a new line."
+
+            # Is it the simple case with no comments between the
+            # old and new desired locations and only a single following
+            # whitespace?
+            reflow_logger.debug("  Trailing Easy Case")
+            # Strip newlines from the next point. Apply the indent to
+            # the previous point.
+            new_results, prev_point = prev_point.indent_to(
+                next_point.get_indent() or "",
+                before=elem_buff[loc.prev.adj_pt_idx + 1].segments[0],
+            )
+            new_results, next_point = next_point.respace_point(
+                cast(ReflowBlock, elem_buff[loc.next.adj_pt_idx - 1]),
+                cast(ReflowBlock, elem_buff[loc.next.adj_pt_idx + 1]),
+                root_segment=root_segment,
+                lint_results=new_results,
+                strip_newlines=True,
+            )
+
+            # Update the points in the buffer
+            elem_buff[loc.prev.adj_pt_idx] = prev_point
+            elem_buff[loc.next.adj_pt_idx] = next_point
+
+        elif loc.line_position == "trailing":
+            if elem_buff[loc.next.newline_pt_idx].num_newlines():
+                # We're good, it's already trailing.
+                continue
+
+            # Generate the text for any issues.
+            pretty_name = loc.pretty_target_name()
+            desc = f"The {pretty_name} should always be at the end of a line."
+
+            # Is it the simple case with no comments between the
+            # old and new desired locations and only one previous newline?
+            reflow_logger.debug("  Leading Easy Case")
+            # Simple case. No comments.
+            # Strip newlines from the previous point. Apply the indent
+            # to the next point.
+            new_results, next_point = next_point.indent_to(
+                prev_point.get_indent() or "",
+                after=elem_buff[loc.next.adj_pt_idx - 1].segments[-1],
+            )
+            new_results, prev_point = prev_point.respace_point(
+                cast(ReflowBlock, elem_buff[loc.prev.adj_pt_idx - 1]),
+                cast(ReflowBlock, elem_buff[loc.prev.adj_pt_idx + 1]),
+                root_segment=root_segment,
+                lint_results=new_results,
+                strip_newlines=True,
+            )
+
+            # Update the points in the buffer
+            elem_buff[loc.prev.adj_pt_idx] = prev_point
+            elem_buff[loc.next.adj_pt_idx] = next_point
+
+        elif loc.line_position == "alone":
+            # If we get here we can assume that the element is currently
+            # either leading or trailing and needs to be moved onto its
+            # own line.
+
+            # Generate the text for any issues.
+            pretty_name = loc.pretty_target_name()
+            desc = (
+                f"The {pretty_name} should always have a line break "
+                "both before and after."
+            )
+
+            # First handle the following newlines first (easy).
+            if not elem_buff[loc.next.newline_pt_idx].num_newlines():
+                reflow_logger.debug("  Found missing newline after in alone case")
+                new_results, next_point = next_point.indent_to(
+                    prev_point.get_indent() or "",
+                    after=elem_buff[loc.next.adj_pt_idx - 1].segments[-1],
+                )
+                # Update the point in the buffer
+                elem_buff[loc.next.adj_pt_idx] = next_point
+
+            # Then handle newlines before. (hoisting past comments if needed).
+            if not elem_buff[loc.prev.adj_pt_idx].num_newlines():
+                reflow_logger.debug("  Found missing newline before in alone case")
+                # NOTE: In the case that there are comments _after_ the
+                # target, they will be moved with it. This might break things
+                # but there isn't an unambiguous way to do this, because we
+                # can't be sure what the comments are referring to.
+                # Given that, we take the simple option.
+                new_results, prev_point = prev_point.indent_to(
+                    next_point.get_indent() or "",
+                    before=elem_buff[loc.prev.adj_pt_idx + 1].segments[0],
+                )
+                # Update the point in the buffer
+                elem_buff[loc.prev.adj_pt_idx] = prev_point
+
+        else:
+            raise NotImplementedError(  # pragma: no cover
+                f"Unexpected line_position config: {loc.line_position}"
+            )
+
+        # Consolidate results and consume fix buffer
+        lint_results.append(
+            LintResult(
+                loc.target,
+                fixes=fixes_from_results(new_results) + fixes,
+                description=desc,
+            )
+        )
+        fixes = []
+
+    return elem_buff, lint_results
